
/* Chrysalide - Outil d'analyse de fichiers binaires
 * virtual.h - équivalent Python du fichier "analysis/blocks/virtual.c"
 *
 * Copyright (C) 2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "virtual.h"


#include <pygobject.h>


#include <analysis/blocks/virtual.h>


#include "../block.h"
#include "../../helpers.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_virtual_block_type(void)
{
    static PyMethodDef py_virtual_block_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_virtual_block_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_virtual_block_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.analysis.blocks.VirtualBlock",

        .tp_flags       = Py_TPFLAGS_DEFAULT,

        .tp_doc         = "PyChrysalide virtual block",

        .tp_methods     = py_virtual_block_methods,
        .tp_getset      = py_virtual_block_getseters,

    };

    return &py_virtual_block_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide...blocks.VirtualBlock'.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool register_python_virtual_block(PyObject *module)
{
    PyTypeObject *py_virtual_block_type;    /* Type Python 'VirtualBlock'  */
    PyObject *dict;                         /* Dictionnaire du module      */

    py_virtual_block_type = get_python_virtual_block_type();

    dict = PyModule_GetDict(module);

    if (!register_class_for_pygobject(dict, G_TYPE_VIRTUAL_BLOCK,
                                      py_virtual_block_type, get_python_instr_block_type()))
        return false;

    return true;

}
